-- FIXME need caps for Critical Magnitude and Critical Defence

local version = 10

-- version history:
-- 10	new version numbering to match LoTRO updates
--	updated agility, will, fate, critical defence, champions, wardens
--	switched hunters to ranged offence, for cases where ranged and melee differ
--	added critical magnitude
--	fixed high offence (which I can't test as I have crummy gear)
-- 0.92 fixed a stupid error in ResetWindow that I introduced in 0.91
-- 0.91 removed "/statcalc hide", added "/statcalc toggle"
--	added German translation, thanks to Amethadan
-- 0.9  initial release

-- Big thanks to Alad and the contributors to this thread:
-- https://forums.lotro.com/showthread.php?424066-Combat-stat-changes-on-Isengard-release.-Mitigation-and-Offence.

-- Feel free to borrow or fork code, unless you're a jerk.

import "Turbine.Gameplay"
import "Turbine.UI.Lotro"
import "PolymniePlugins.StatCalcTranslate"

-- Change this from nil to "en" or "de" to override locale detection
local Language = nil

if not Language then
	Language = Turbine.Engine.GetLanguage()
	if Language == Turbine.Language.German then
		Language = "de"
	else
		Language = "en"
	end
end

-------------------------------------------------------------------------------
local MyChar = Turbine.Gameplay.LocalPlayer:GetInstance()
local MyAttributes = MyChar:GetAttributes()
local MyClass = MyChar:GetClass()
local MyLevel

-------------------------------------------------------------------------------
function CheckClass()
	-- Freep
	if MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Captain or
	MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian or
	MyClass == Turbine.Gameplay.Class.Hunter or
	MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper or
	MyClass == Turbine.Gameplay.Class.Warden then
		return

	-- Creep
	elseif MyClass == Turbine.Gameplay.Class.BlackArrow or
	MyClass == Turbine.Gameplay.Class.Defiler or
	MyClass == Turbine.Gameplay.Class.Reaver or
	MyClass == Turbine.Gameplay.Class.Stalker or
	MyClass == Turbine.Gameplay.Class.WarLeader or
	MyClass == Turbine.Gameplay.Class.Weaver then
		Turbine.Shell.WriteLine(translate("Warning: StatCalc is broken for Monster Play classes.", Language))

	-- Session
	else
		Turbine.Shell.WriteLine(translate("Warning: StatCalc needs improvement for Session Play classes.", Language))
	end
end

-------------------------------------------------------------------------------
function round(number, zeroes)
	local zeroes = zeroes or 0
	return math.floor(number * (10 ^ zeroes) + .5) / (10 ^ zeroes)
end

-------------------------------------------------------------------------------
function RatingsToPercent(RperL, K, Xo, Yo)
	return Yo + 1 / (1 + K / (RperL - Xo))
end

-------------------------------------------------------------------------------
-- Physical Mitigation
-- Tactical Mitigation
function Mitigation(R, L)
	local percent

	if MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper then

		-- break: 20% @ 37.5 R/L
		if (R / L) > 37.5 then
			percent = RatingsToPercent(R / L, 350, 37.5, 0.2)
		else
			percent = RatingsToPercent(R / L, 150, 0, 0)
		end
		-- maximum: 40% @ 125 R/L
		return math.min(0.4, percent)

	elseif MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Hunter or
	MyClass == Turbine.Gameplay.Class.Warden then

		-- break: 20% @ 37.5 R/L
		if (R / L) > 37.5 then
			percent = RatingsToPercent(R / L, 350, 37.5, 0.2)
		else
			percent = RatingsToPercent(R / L, 150, 0, 0)
		end
		-- maximum: 50% @ 187.5 R/L
		return math.min(0.5, percent)

	elseif MyClass == Turbine.Gameplay.Class.Captain or
	MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian then

		-- break: 10% @ 50/3 R/L
		if (R / L) > (50 / 3) then
			percent = RatingsToPercent(R / L, 150, 50 / 3, 0.1)
		else
			percent = RatingsToPercent(R / L, 150, 0, 0)
		end
		-- maximum: 70% @ 725/3 R/L
		return math.min(0.7, percent)

	else  -- Session Play uses the same formulae as Heavy Mitigation (I think)

		-- break: 10% @ 50/3 R/L
		if (R / L) > (50 / 3) then
			percent = RatingsToPercent(R / L, 150, 50 / 3, 0.1)
		else
			percent = RatingsToPercent(R / L, 150, 0, 0)
		end
		-- maximum: 70% @ 725/3 R/L
		return math.min(0.7, percent)

	end
end

-- Physical Offence
-- Ranged Offence
-- Tactical Offence
function Offence(R, L)
	local segment, remainder = math.modf((R - 1) / L / 75)

	local percent = RatingsToPercent(R / L, 300, segment * 75, segment / 5)

	-- no maximum
	return percent
end

-- Critical Hit
-- Incoming Healing
-- Block
-- Parry
-- Evade
-- Melee Defence
-- Ranged Defence
-- Tactical Defence
function Avoidance(R, L)
	local percent
	-- break: 15% @ 70 R/L
	if (R / L) > 70 then
		percent = RatingsToPercent(R / L, 2380 / 3, 70, 0.15)
	else
		percent = RatingsToPercent(R / L, 1190 / 3, 0, 0)
	end
	-- maximum: 25% @ 4270/27 R/L

	return math.min(0.25, percent)
end

-- Devastating Hit
-- Devastate Defence
-- Partial Block
-- Partial Parry
-- Partial Evade
function PartialAvoidance(R, L)
	local percent
	percent = RatingsToPercent(R / L, 1330, 0, 0)
	-- maximum: 10% @ 1330/9 R/L
	return math.min(0.1, percent)
end

-- Finesse
-- Resistance
-- Outgoing Healing
function Resistance(R, L)
	local percent
	-- break: 30% @ 170 R/L
	if (R / L) > 170 then
		percent = RatingsToPercent(R / L, 2380 / 3, 170, 0.3)
	else
		percent = RatingsToPercent(R / L, 1190 / 3, 0, 0)
	end
	-- maximum: 50% @ 1105/3 R/L
	return math.min(0.5, percent)
end

-- Critical Defence
function CriticalDefence(R, L)
	local percent
	percent = RatingsToPercent(R / L, 100, 0, 0)
	-- maximum: FIXME
--	return math.min(1, percent)
	return percent
end

-- Critical Magnitude
function CriticalMagnitude(R, L)
	local percent
	percent = RatingsToPercent(R / L, 300, 0, 0)
	-- maximum: FIXME
--	return math.min(1, percent)
	return percent
end

-------------------------------------------------------------------------------
local Depth = 30
local RowHeight = 20
local WindowWidth = 490
local WindowHeight = RowHeight * 33

CalcWindow = Turbine.UI.Lotro.Window()
CalcWindow:SetPosition((Turbine.UI.Display:GetWidth() - WindowWidth) / 2, 0)
CalcWindow:SetSize(WindowWidth, WindowHeight)
CalcWindow:SetText("StatCalc "..version)
CalcWindow:SetVisible(true)

--local Grid = {}
--for i = 0, WindowWidth, 10 do
--	Grid[i] = Turbine.UI.Label()
--	Grid[i]:SetBackColor(Turbine.UI.Color(0.5, 0, 0.5))
--	Grid[i]:SetMouseVisible(false)
--	Grid[i]:SetParent(CalcWindow)
--	Grid[i]:SetPosition(i, 0)
--	Grid[i]:SetSize(1, WindowHeight)
--end

function MakeLabel(Left, Top, Width, Height, Text, Alignment)
	local TempLabel = Turbine.UI.Label()
	TempLabel:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempLabel:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1))
	TempLabel:SetParent(CalcWindow)
	TempLabel:SetPosition(Left, Top)
	TempLabel:SetSize(Width, Height)
	TempLabel:SetMouseVisible(false)
	if Text then
		TempLabel:SetText(Text)
	end

	if Alignment then
		TempLabel:SetTextAlignment(Alignment)
	end

	return TempLabel
end

function MakeTextbox(Left, Top, Width, Height)
	local TempTextbox = Turbine.UI.Lotro.TextBox()
	TempTextbox:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempTextbox:SetParent(CalcWindow)
	TempTextbox:SetPosition(Left, Top)
	TempTextbox:SetSize(Width, Height)
	TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)

	TempTextbox.FocusGained = function()
		TempTextbox:SetBackColor(Turbine.UI.Color.Blue)
		TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
	end
	TempTextbox.FocusLost = function()
		TempTextbox:SetBackColor(Turbine.UI.Color.Black)
		TempTextbox:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)
	end

	return TempTextbox
end

function MakeButton(Left, Top, Width, Height, Text)
	local TempButton = Turbine.UI.Lotro.Button()
	TempButton:SetParent(CalcWindow)
	TempButton:SetPosition(Left, Top)
	TempButton:SetSize(Width, Height)
	TempButton:SetFont(Turbine.UI.Lotro.Font.Verdana16)
	TempButton:SetText(Text)
	return TempButton
end

local Rows = {}

function MakeRow(stat, percent, substat)
	Rows[stat] = {}

	Rows[stat].name = MakeLabel(10, Depth, 180, RowHeight - 1, translate(stat, Language))
	if substat then
		Rows[stat].name:SetForeColor(Turbine.UI.Color(0.7, 0.7, 0.7))
	end

	-- Character's current stats
	if not substat then
		Rows[stat].base = MakeLabel(200, Depth, 40, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)
	end

	-- Character's current percentages
	if percent then
		Rows[stat].baseP = MakeLabel(250, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)
	end

	if not substat then
		-- User-supplied stat-adjustment
		Rows[stat].delta = MakeTextbox(310, Depth, 60, RowHeight - 1)

		-- Resulting stats
		Rows[stat].result = MakeLabel(380, Depth, 40, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)
	end

	-- Resulting percentages
	if percent then
		Rows[stat].resultP = MakeLabel(430, Depth, 50, RowHeight - 1, nil, Turbine.UI.ContentAlignment.TopRight)
	end

	-- increment
	Depth = Depth + RowHeight
end

MakeRow("Maximum Morale")
MakeRow("in-Combat Morale Regen")
MakeRow("non-Combat Morale Regen")
MakeRow("Maximum Power")
MakeRow("in-Combat Power Regen")
MakeRow("non-Combat Power Regen")
MakeRow("Armour")
MakeRow("Might")
MakeRow("Agility")
MakeRow("Vitality")
MakeRow("Will")
MakeRow("Fate")
MakeRow("Critical Hit", true)
MakeRow("Devastating Hit", true, true)
MakeRow("Critical Magnitude", true, true)
MakeRow("Finesse", true)
MakeRow("Physical Mastery", true)
MakeRow("Tactical Mastery", true)
MakeRow("Outgoing Healing", true, true)
MakeRow("Resistance", true)
MakeRow("Critical Defence", true)
MakeRow("Incoming Healing", true)
MakeRow("Block", true)
MakeRow("Partial Block", true, true)
MakeRow("Parry", true)
MakeRow("Partial Parry", true, true)
MakeRow("Evade", true)
MakeRow("Partial Evade", true, true)
MakeRow("Physical Mitigation", true)
MakeRow("Tactical Mitigation", true)

local ResetButton = MakeButton(280, Depth, 55, 20, "reset")
ResetButton.Click = function()
	ResetWindow()
end

local UpdateButton = MakeButton(340, Depth, 65, 20, "update")
UpdateButton.Click = function()
	UpdateWindow()
end

-------------------------------------------------------------------------------
function ValidateTextbox(box)
	local input
	input = tonumber(box:GetText())
	if not input then
		input = 0
		box:SetText(nil)
	elseif input == 0 then
		box:SetText(nil)
	else
		box:SetText(input)
	end
	return input
end

-------------------------------------------------------------------------------
function UpdateRow(row, base, result, untouched, fPercent, child)
	local colour
	if untouched then
		colour = Turbine.UI.Color.White
	elseif result < base then
		colour = Turbine.UI.Color.Red
	elseif result > base then
		colour = Turbine.UI.Color.Lime
	else
		colour = Turbine.UI.Color.Yellow
	end

	if not child then
		Rows[row].base:SetText(string.format("%.0f", base))
		Rows[row].result:SetText(string.format("%.0f", result))
		Rows[row].result:SetForeColor(colour)
	end

	if fPercent then
		Rows[row].baseP:SetText(string.format("%.1f%%", fPercent(base, MyLevel) * 100))
		Rows[row].resultP:SetText(string.format("%.1f%%", fPercent(result, MyLevel) * 100))
		Rows[row].resultP:SetForeColor(colour)
	end
end

-------------------------------------------------------------------------------
function UpdateWindow()
	MyLevel = MyChar:GetLevel()

	-- get the character's current stats
	local morale1 = MyChar:GetMaxMorale()
	local icmr1 = MyAttributes:GetInCombatMoraleRegeneration() * 60 -- * 60
	local ncmr1 = MyAttributes:GetOutOfCombatMoraleRegeneration() * 60 -- * 60
	local power1 = MyChar:GetMaxPower()
	local icpr1 = MyAttributes:GetInCombatPowerRegeneration() * 60 -- * 60
	local ncpr1 = MyAttributes:GetOutOfCombatPowerRegeneration() * 60 -- * 60
	local armour1 = MyAttributes:GetArmor()
	local might1 = MyAttributes:GetMight()
	local agility1 = MyAttributes:GetAgility()
	local vitality1 = MyAttributes:GetVitality()
	local will1 = MyAttributes:GetWill()
	local fate1 = MyAttributes:GetFate()
	local crit1 = MyAttributes:GetBaseCriticalHitChance()
--	if MyAttributes:GetMeleeCriticalHitChance() > crit1 then
--		Turbine.Shell.WriteLine("StatCalc warning: Melee Critical Rating is higher than Base Critical Rating.")
--	end
--	if MyAttributes:GetRangeCriticalHitChance() > crit1 then
--		Turbine.Shell.WriteLine("StatCalc warning: Ranged Critical Rating is higher than Base Critical Rating.")
--	end
--	if MyAttributes:GetTacticalCriticalHitChance() > crit1 then
--		Turbine.Shell.WriteLine("StatCalc warning: Tactical Critical Rating is higher than Base Critical Rating.")
--	end

	local finesse1 = MyAttributes:GetFinesse()


	local physmas1
	if MyClass == Turbine.Gameplay.Class.Hunter then
		physmas1 = MyAttributes:GetRangeDamage()
	else
		physmas1 = MyAttributes:GetMeleeDamage()
	end

	local tacmas1 = MyAttributes:GetTacticalDamage()
	local resist1 = MyAttributes:GetBaseResistance()
	local critdef1 = MyAttributes:GetBaseCriticalHitAvoidance()
	local incoming1 = MyAttributes:GetIncomingHealing()
	local block1 = MyAttributes:GetBlock()
	local parry1 = MyAttributes:GetParry()
	local evade1 = MyAttributes:GetEvade()
	local physmit1 = MyAttributes:GetCommonMitigation() -- not MyAttributes:GetPhysicalMitigation()
	local tacmit1 = MyAttributes:GetTacticalMitigation()

	-- validate user's input
	local morale2 = ValidateTextbox(Rows["Maximum Morale"].delta)
	local icmr2 = ValidateTextbox(Rows["in-Combat Morale Regen"].delta)
	local ncmr2 = ValidateTextbox(Rows["non-Combat Morale Regen"].delta)
	local power2 = ValidateTextbox(Rows["Maximum Power"].delta)
	local icpr2 = ValidateTextbox(Rows["in-Combat Power Regen"].delta)
	local ncpr2 = ValidateTextbox(Rows["non-Combat Power Regen"].delta)
	local armour2 = ValidateTextbox(Rows["Armour"].delta)
	local might2 = ValidateTextbox(Rows["Might"].delta)
	local agility2 = ValidateTextbox(Rows["Agility"].delta)
	local vitality2 = ValidateTextbox(Rows["Vitality"].delta)
	local will2 = ValidateTextbox(Rows["Will"].delta)
	local fate2 = ValidateTextbox(Rows["Fate"].delta)
	local crit2 = ValidateTextbox(Rows["Critical Hit"].delta)
	local finesse2 = ValidateTextbox(Rows["Finesse"].delta)
	local physmas2 = ValidateTextbox(Rows["Physical Mastery"].delta)
	local tacmas2 = ValidateTextbox(Rows["Tactical Mastery"].delta)
	local resist2 = ValidateTextbox(Rows["Resistance"].delta)
	local critdef2 = ValidateTextbox(Rows["Critical Defence"].delta)
	local incoming2 = ValidateTextbox(Rows["Incoming Healing"].delta)
	local block2 = ValidateTextbox(Rows["Block"].delta)
	local parry2 = ValidateTextbox(Rows["Parry"].delta)
	local evade2 = ValidateTextbox(Rows["Evade"].delta)
	local physmit2 = ValidateTextbox(Rows["Physical Mitigation"].delta)
	local tacmit2 = ValidateTextbox(Rows["Tactical Mitigation"].delta)

	-- calculating resulting stats
	local morale3
	if MyClass == Turbine.Gameplay.Class.Guardian or
	MyClass == Turbine.Gameplay.Class.Warden then
		morale3  = morale1   + morale2   + vitality2 * 5
	else
		morale3  = morale1   + morale2   + vitality2 * 3
	end

	local icmr3     = icmr1     + icmr2     + fate2 * 1.5
	local ncmr3     = ncmr1     + ncmr2     + vitality2 * 7.2
	local power3    = power1    + power2
	local icpr3     = icpr1     + icpr2     + fate2 * 5.52
	local ncpr3     = ncpr1     + ncpr2     + fate2 * 24
	local armour3   = armour1   + armour2
	local might3    = might1    + might2
	local agility3  = agility1  + agility2
	local vitality3 = vitality1 + vitality2
	local will3     = will1     + will2
	local fate3     = fate1     + fate2
	local crit3     = crit1     + crit2     + agility2 + fate2 * 2.5
	local finesse3  = finesse1  + finesse2

	local physmas3
	if MyClass == Turbine.Gameplay.Class.Captain or
	MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian then
		physmas3 = physmas1  + physmas2 + might2 * 10
	elseif MyClass == Turbine.Gameplay.Class.Warden then
		physmas3 = physmas1  + physmas2 + might2 * 10 + agility2 * 10
	elseif MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Hunter then
		physmas3 = physmas1  + physmas2 + agility2 * 10
	elseif MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper then
		physmas3 = physmas1  + physmas2 + will2 * 10
	else -- Session Play
		physmas3 = physmas1  + physmas2
	end

	local tacmas3
	if MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian or
	MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Hunter then
		tacmas3  = tacmas1   + tacmas2  + will2 * 5
	elseif MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper then
		tacmas3  = tacmas1   + tacmas2  + will2 * 10
	elseif MyClass == Turbine.Gameplay.Class.Captain then
		tacmas3  = tacmas1   + tacmas2  + might2 * 10

	elseif MyClass == Turbine.Gameplay.Class.Warden then
		tacmas3  = tacmas1   + tacmas2  + might2 * 5 + agility2 * 5

	else -- Session Play
		tacmas3  = tacmas1   + tacmas2
	end

	local resist3   = resist1   + resist2   + vitality2 * 2 + will2 * 2
	local critdef3  = critdef1  + critdef2
	local incoming3 = incoming1 + incoming2

	local block3, parry3
	if MyClass == Turbine.Gameplay.Class.Champion then
		block3    = block1    + block2
		parry3    = parry1    + parry2    + might2 * 6 + agility2 * 2
	else
		block3    = block1    + block2    + might2 * 4
		parry3    = parry1    + parry2    + might2 * 2 + agility2 * 2
	end

	local evade3    = evade1    + evade2    + agility2 * 4
	local physmit3  = physmit1  + physmit2  + armour2 + might2 * 2 + vitality2 * 2
	local tacmit3   = tacmit1   + tacmit2   + armour2 / 5 + vitality2 * 4 + will2 * 2

	-- update display
	UpdateRow("Maximum Morale", morale1, morale3, morale2 == 0 and vitality2 == 0)
	UpdateRow("in-Combat Morale Regen", icmr1, icmr3, icmr2 == 0 and fate2 == 0)
	UpdateRow("non-Combat Morale Regen", ncmr1, ncmr3, ncmr2 == 0 and vitality2 == 0)
	UpdateRow("Maximum Power", power1, power3, power2 == 0)
	UpdateRow("in-Combat Power Regen", icpr1, icpr3, icpr2 == 0 and fate2 == 0)
	UpdateRow("non-Combat Power Regen", ncpr1, ncpr3, ncpr2 == 0 and fate2 == 0)
	UpdateRow("Armour", armour1, armour3, armour2 == 0)
	UpdateRow("Might", might1, might3, might2 == 0)
	UpdateRow("Agility", agility1, agility3, agility2 == 0)
	UpdateRow("Vitality", vitality1, vitality3, vitality2 == 0)
	UpdateRow("Will", will1, will3, will2 == 0)
	UpdateRow("Fate", fate1, fate3, fate2 == 0)
	UpdateRow("Critical Hit", crit1, crit3, crit2 == 0 and agility2 == 0 and fate2 == 0, Avoidance)
	UpdateRow("Devastating Hit", crit1, crit3, crit2 == 0 and agility2 == 0 and fate2 == 0, PartialAvoidance, true)
	UpdateRow("Critical Magnitude", crit1, crit3, crit2 == 0 and agility2 == 0 and fate2 == 0, CriticalMagnitude, true)
	UpdateRow("Finesse", finesse1, finesse3, finesse2 == 0, Resistance)

	if MyClass == Turbine.Gameplay.Class.Captain or
	MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian then
		UpdateRow("Physical Mastery", physmas1, physmas3, physmas2 == 0 and might2 == 0, Offence)

	elseif MyClass == Turbine.Gameplay.Class.Warden then
		UpdateRow("Physical Mastery", physmas1, physmas3, physmas2 == 0 and might2 == 0 and agility2 == 0, Offence)

	elseif MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Hunter then
		UpdateRow("Physical Mastery", physmas1, physmas3, physmas2 == 0 and agility2 == 0, Offence)

	elseif MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper then
		UpdateRow("Physical Mastery", physmas1, physmas3, physmas2 == 0 and will2 == 0, Offence)

	else -- Session Play
		UpdateRow("Physical Mastery", physmas1, physmas3, physmas2 == 0, Offence)
	end

	if MyClass == Turbine.Gameplay.Class.Burglar or
	MyClass == Turbine.Gameplay.Class.Champion or
	MyClass == Turbine.Gameplay.Class.Guardian or
	MyClass == Turbine.Gameplay.Class.Hunter or
	MyClass == Turbine.Gameplay.Class.LoreMaster or
	MyClass == Turbine.Gameplay.Class.Minstrel or
	MyClass == Turbine.Gameplay.Class.RuneKeeper then
		UpdateRow("Tactical Mastery", tacmas1, tacmas3, tacmas2 == 0 and will2 == 0, Offence)
		UpdateRow("Outgoing Healing", tacmas1, tacmas3, tacmas2 == 0 and will2 == 0, Resistance, true)

	elseif MyClass == Turbine.Gameplay.Class.Captain then
		UpdateRow("Tactical Mastery", tacmas1, tacmas3, tacmas2 == 0 and might2 == 0, Offence)
		UpdateRow("Outgoing Healing", tacmas1, tacmas3, tacmas2 == 0 and might2 == 0, Resistance, true)

	elseif MyClass == Turbine.Gameplay.Class.Warden then
		UpdateRow("Tactical Mastery", tacmas1, tacmas3, tacmas2 == 0 and might2 == 0 and agility2 == 0, Offence)
		UpdateRow("Outgoing Healing", tacmas1, tacmas3, tacmas2 == 0 and might2 == 0 and agility2 == 0, Resistance, true)

	else -- Session Play
		UpdateRow("Tactical Mastery", tacmas1, tacmas3, tacmas2 == 0, Offence)
		UpdateRow("Outgoing Healing", tacmas1, tacmas3, tacmas2 == 0, Resistance, true)
	end

	UpdateRow("Resistance", resist1, resist3, resist2 == 0 and will2 == 0 and vitality2 == 0, Resistance)
	UpdateRow("Critical Defence", critdef1, critdef3, critdef2 == 0, CriticalDefence)
	UpdateRow("Incoming Healing", incoming1, incoming3, incoming2 == 0, Avoidance)

	if MyClass == Turbine.Gameplay.Class.Champion then
		UpdateRow("Block", block1, block3, block2 == 0, Avoidance)
		UpdateRow("Partial Block", block1, block3, block2 == 0, PartialAvoidance, true)
	else
		UpdateRow("Block", block1, block3, block2 == 0 and might2 == 0, Avoidance)
		UpdateRow("Partial Block", block1, block3, block2 == 0 and might2 == 0, PartialAvoidance, true)
	end

	UpdateRow("Parry", parry1, parry3, parry2 == 0 and might2 == 0 and agility2 == 0, Avoidance)
	UpdateRow("Partial Parry", parry1, parry3, parry2 == 0 and might2 == 0 and agility2 == 0, PartialAvoidance, true)
	UpdateRow("Evade", evade1, evade3, evade2 == 0 and agility2 == 0, Avoidance)
	UpdateRow("Partial Evade", evade1, evade3, evade2 == 0 and agility2 == 0, PartialAvoidance, true)
	UpdateRow("Physical Mitigation", physmit1, physmit3, physmit2 == 0 and armour2 == 0 and might2 == 0 and vitality2 == 0, Mitigation)
	UpdateRow("Tactical Mitigation", tacmit1, tacmit3, tacmit2 == 0 and armour2 == 0 and vitality2 == 0 and will2 == 0, Mitigation)
end

-------------------------------------------------------------------------------
function ResetWindow()
	for k, v in pairs(Rows) do
		if v.delta then
			v.delta:SetText(nil)
		end
	end
	UpdateWindow()
end

-------------------------------------------------------------------------------
function UpdateLanguage()
	for k, v in pairs(Rows) do
		v.name:SetText(translate(k, Language))
	end
end

-------------------------------------------------------------------------------
local CommandLine = Turbine.ShellCommand()
CommandLine.Execute = function(sender, cmd, args)
	if args == "toggle" and CalcWindow:IsVisible() then
		CalcWindow:SetVisible(false)
		return
	else
		CalcWindow:SetVisible(true)
	end

	if args == "reset" then
		ResetWindow()
		return
	elseif args == "german" then
		Language = "de"
		UpdateLanguage()
	elseif args == "english" then
		Language = "en"
		UpdateLanguage()
	end

	UpdateWindow()
end

Turbine.Shell.AddCommand("StatCalc", CommandLine)

-------------------------------------------------------------------------------
-- create a dummy window, and update until loaded, at which point set loaded=true and set the unload commands.
local loaded = false
tmpWindow = Turbine.UI.Window()
tmpWindow.Update = function()
	if Plugins["StatCalc"] ~= nil and not loaded then
		loaded = true
		Plugins["StatCalc"].Unload = function()
			Turbine.Shell.RemoveCommand(CommandLine)
		end
		tmpWindow:SetWantsUpdates(false)
		Turbine.Shell.WriteLine("Polymnie's StatCalc plugin loaded.")
		CheckClass()
		UpdateWindow()
	end
end
tmpWindow:SetWantsUpdates(true)
